@@ -1,6 +1,10 @@
 package Reddit::Client;
 
-our $VERSION = '1.244';
+our $VERSION = '1.26';
+# 1.26  crossposts
+# 1.252 Dropped token refresh time to 3550 (1 hour - 50 seconds)
+# 1.251 removed unnecessary passing in of variables to get_token in request
+# 1.25	username can now optionally be passed in for web type apps
 # 1.244 token request is called in static context; if failed it would look for $self->{request_errors} but there was no self
 # 1.243 rolled back change making request_errors default
 # 1.242 request_errors (print the content of errors) is now true by default.
@@ -9,12 +13,6 @@ our $VERSION = '1.244';
 # be pretty amazed if we didn't break anything 
 # 1.23 get_modmail_raw
 #	   get_refresh_token
-# 1.222 Removing unused fields from Comment/Post
-# 1.22 get refresh token functionality
-# 1.21 added spam()
-# 1.21 get modmail. hidden feature because it's a godawful mess
-# 1.202 username is now required for get or create multis unless script auth
-# 1.20 web type apps now supported!
 
 $VERSION = eval $VERSION;
 
@@ -46,7 +44,7 @@ use constant VIEW_HOT                => '';
 use constant VIEW_NEW                => 'new';
 use constant VIEW_CONTROVERSIAL      => 'controversial';
 use constant VIEW_TOP                => 'top';
-use constant VIEW_RISING	          => 'rising';
+use constant VIEW_RISING	         => 'rising';
 use constant VIEW_DEFAULT            => VIEW_HOT;
 
 use constant VOTE_UP                 => 1;
@@ -56,10 +54,11 @@ use constant VOTE_NONE               => 0;
 use constant SUBMIT_LINK             => 'link';
 use constant SUBMIT_SELF       	     => 'self';
 use constant SUBMIT_MESSAGE          => 'message';
+use constant SUBMIT_CROSSPOST        => 'crosspost';
 
-use constant MESSAGES_INBOX	     => 'inbox';
+use constant MESSAGES_INBOX	         => 'inbox';
 use constant MESSAGES_UNREAD	     => 'unread';
-use constant MESSAGES_SENT	     => 'sent';
+use constant MESSAGES_SENT	         => 'sent';
 use constant MESSAGES_MESSAGES       => 'messages';
 use constant MESSAGES_COMMENTREPLIES => 'comments';
 use constant MESSAGES_POSTREPLIES    => 'selfreply';
@@ -99,10 +98,10 @@ use constant API_LINKS_OTHER         => 13;
 use constant API_DEL                 => 14;
 use constant API_MESSAGE             => 15;
 use constant API_COMMENTS_FRONT	     => 16;
-use constant API_COMMENTS	     => 17;
-use constant API_MESSAGES	     => 18;
-use constant API_MARK_READ	     => 19;
-use constant API_MARKALL	     => 20;
+use constant API_COMMENTS	         => 17;
+use constant API_MESSAGES	         => 18;
+use constant API_MARK_READ	         => 19;
+use constant API_MARKALL	         => 20;
 use constant API_MY_SUBREDDITS       => 21;
 use constant API_USER                => 22;
 use constant API_SELECTFLAIR         => 23;
@@ -181,22 +180,25 @@ $API[API_BAN           ] = ['POST', '/r/%s/api/friend'        ];
 #===============================================================================
 
 use fields (
-    'modhash',      # store session modhash
-    'cookie',       # store user cookie
-    'session_file', # path to session file
+    'modhash',      # No longer used. stored session modhash
+    'cookie',       # No longer used. stored user cookie
+    'session_file', # No longer used. path to session file
     'user_agent',   # user agent string
     'token',		# oauth authorization token
     'tokentype',    # unused but saved for reference
     'last_token',	# time last token was acquired
-    'client_id',	# These 4 values saved for automatic token refreshing
-    'secret',       
-	'username',
-	'password',
-	'request_errors',# print request errors
-	'print_response',# print entire response
-	'print_request', # print entire request
-	'refresh_token', # oauth refresh token
-	'auth_type',
+    'client_id',	# always required
+    'secret',       # always required
+	'username',		# now optional for web apps
+	'password',		# script apps only
+	'request_errors',		# print request errors
+	'print_request_errors',	# print request errors, deprecated
+	'print_response',		# print response content, deprecated
+	'print_response_content',# print response content
+	'print_request', 		# print entire request
+	'refresh_token', 		# oauth refresh token
+	'auth_type',			# 'script' or 'webapp'
+	'debug',
 );
 
 sub new {
@@ -208,12 +210,13 @@ sub new {
 	}
 	$self->{user_agent} 	= $param{user_agent};
 	$self->{request_errors} = $param{print_request_errors} || $param{request_errors} || 0;
-	$self->{print_response} = $param{print_response} || 0;
+	$self->{print_response} = $param{print_response} || $param{print_response_conent} || 0;
 	$self->{print_request}  = $param{print_request} || 0;
+	$self->{debug}			= $param{debug}	|| 0;
 
-	if ($param{username} or $param{password}) {
-		if (!$param{username} or !$param{password}) {
-			croak "If username or  password are provided, both are required.";
+	if ($param{password}) {
+		if (!$param{username}) {
+			croak "if password is provided, username is required.";
 		} elsif (!$param{client_id} or !$param{secret}) {
 			croak "client_id and secret are required for authorized apps.";
 		} else {
@@ -231,7 +234,9 @@ sub new {
 		$self->{auth_type} 	= 'webapp';
 		$self->{client_id}	= $param{client_id};
 		$self->{secret} 	= $param{secret};
-		$self->{refresh_token} 	= $param{refresh_token};
+		$self->{refresh_token}= $param{refresh_token};
+		# will this break anything?
+		$self->{username}	= $param{username} if $param{username};
 
 		$self->get_token();
 	} else {
@@ -258,13 +263,10 @@ sub version {
 sub request {
     my ($self, $method, $path, $query, $post_data) = @_;
 
-	if (!$self->{last_token} || $self->{last_token} <= time - 3595) {
-		$self->get_token(
-			client_id	=> $self->{client_id}, 
-			secret		=> $self->{secret}, 
-			username	=> $self->{username}, 
-			password	=> $self->{password},
-		);
+	if (!$self->{last_token} or $self->{last_token} <= time - 3550) {
+		#if ($self->{debug}) {print "getting new token\n";}
+		# passing in username, pass, client_id, secret here did nothing
+		$self->get_token();
 	}
 
     # Trim leading slashes off of the path
@@ -279,11 +281,11 @@ sub request {
         post_data  => $post_data,
         modhash    => $self->{modhash},
         cookie     => $self->{cookie},
-	token	   => $self->{token},
-	tokentype  => $self->{tokentype},
-	request_errors=> $self->{request_errors},
-	print_response=> $self->{print_response},
-	print_request=>  $self->{print_request},
+		token	   => $self->{token},
+		tokentype  => $self->{tokentype},
+		request_errors=> $self->{request_errors},
+		print_response=> $self->{print_response},
+		print_request=>  $self->{print_request},
     );
 
     return $request->send;
@@ -293,6 +295,7 @@ sub get_token {
 	my ($self, %param) = @_;
 
 	# let people set auth things here. this was stupid to allow.
+	# these all set $self properties then continue as normal.
 	if ($param{username} or $param{password}) {
 		die "get_token: if username or password are provided, all 4 script-type authentication arguments (username, password, client_id, secret) are required." unless $param{username} and $param{password} and $param{client_id} and $param{secret}; 
 
@@ -304,8 +307,8 @@ sub get_token {
 
 	} elsif ($param{refresh_token}) {
 		$self->{auth_type} 	= 'webapp';
-		$self->{client_id}	= $param{client_id} || $self->{client_id} || die "get_token: 'client_id' must be set, either as a parameter to get_token or when instantiating the Reddit::Client object. ";
-		$self->{secret} 	= $param{secret} || $self->{secret} || die "get_token: 'secret' must be set, either as a parameter to get_token or when instantiating the Reddit::Client object. ";
+		$self->{client_id}	= $param{client_id} || $self->{client_id} || die "get_token: 'client_id' must be set, either as a parameter to get_token or when instantiating the Reddit::Client object.";
+		$self->{secret} 	= $param{secret} || $self->{secret} || die "get_token: 'secret' must be set, either as a parameter to get_token or when instantiating the Reddit::Client object.";
 		$self->{refresh_token} 	= $param{refresh_token};
 	}
 
@@ -972,6 +975,7 @@ sub delete {
 
 sub submit_link {
     my ($self, %param) = @_;
+	# why is sub allowed to be empty?
     my $subreddit = $param{subreddit} || $param{sub} || '';
     my $title     = $param{title}     || croak 'Expected "title"';
     my $url       = $param{url}       || croak 'Expected "url"';
@@ -994,9 +998,35 @@ sub submit_link {
     return $result->{data}{name};
 }
 
+sub submit_crosspost {
+    my ($self, %param) = @_;
+	# why is subreddit allowed to be empty?
+    my $subreddit = $param{subreddit} || $param{sub} || die "expected 'subreddit'\n";
+    my $title     = $param{title}     || die "Expected 'title'\n";
+	my $source_id = $param{source_id} || die "Expected 'source_id'\n";
+	$source_id = "t3_$source_id" if lc substr($source_id, 0, 3) ne 't3_';
+    #my $url       = $param{url}       || croak 'Expected "url"';
+    my $replies = exists $param{inbox_replies} ? ($param{inbox_replies} ? "true" : "false") : "true";
+    my $repost = exists $param{repost} ? ($param{repost} ? "true" : "false") : "false";
+
+    $subreddit = subreddit($subreddit);
+
+    my $result = $self->api_json_request(api => API_SUBMIT, data => {
+        title       		=> $title,
+        #url         => $url,
+		crosspost_fullname 	=> $source_id, 
+        sr          		=> $subreddit,
+        kind        		=> SUBMIT_CROSSPOST,
+		sendreplies 		=> $replies,
+		resubmit    		=> $repost,
+    });
+
+    return $result->{data}{name};
+}
+
 sub submit_text {
     my ($self, %param) = @_;
-    my $subreddit = $param{subreddit} || $param{sub} || '';
+    my $subreddit = $param{subreddit} || $param{sub} || die "expected 'subreddit'\n";
     my $title     = $param{title}     || croak 'Expected "title"';
     my $text      = $param{text}      || croak 'Expected "text"';
     # true and false have to be the strings "true" or "false"
@@ -1333,7 +1363,7 @@ sub create_multi {
 sub get_multi {
 	my ($self, %param) = @_;
 	my $name	= $param{name} || croak "expected 'name'";
-	my $username 	= $param{user} || $self->{username};
+	my $username= $param{user} || $param{username} || $self->{username} || die "'username' is required.\n";
 	my $expand	= $param{expand} ? '?expand_srs=true' : '';
 
 	my $result = $self->api_json_request( 
@@ -1342,10 +1372,9 @@ sub get_multi {
 	);
 
 	# The result looks like a Subreddit object, but is not.
-	# By returning just the data we lose the 'kind' key,
+	# By returning just the data we lose only the 'kind' key,
 	# which is just the string "LabeledMulti"
 	return $result->{data};
-	return $result;
 }
 
 sub delete_multi {
