@@ -148,11 +148,12 @@ sub token_request {
 	if ($res->is_success) {
 		return $res->decoded_content;
 	} else {
-		if ($self->{request_errors}) {
-			croak "Request error: HTTP ".$res->status_line .", Content: $res->{_content}";
-		} else {
+		# this is sometimes called in static context
+		#if ($self->{request_errors}) {
+		#	croak "Request error: HTTP ".$res->status_line .", Content: $res->{_content}";
+		#} else {
 			croak sprintf("Request error: HTTP %s", $res->status_line);
-		}
+		#}
 		#croak sprintf('Request error: HTTP %s', $res->status_line);
 	}
 }
